// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 - 2006 Simon Goodall

package WFUT;

import java.io.*;
import java.util.*;
import java.net.*;

class FileObjectWriter {

  public static void writeFileHeader(final FileWriter output, final String channel) throws IOException {
    output.write("<?xml version=\"1.0\"?>\n");
    output.write("<?xml-stylesheet type=\"text/xsl\" href=\"filelist.xsl\"?>\n");
    output.write("<fileList dir=\"" + channel +"\">\n");
  }
  
  public static void writeFileObject(final FileWriter output, final FileObject fo) throws IOException {
    try {
    String filename = URLEncoder.encode(fo.getFilename(), "UTF-8");
    output.write("  <file filename=\"" + filename + "\" version=\"" + Integer.toString(fo.getVersion()) + "\" crc32=\"" + Long.toString(fo.getCRC32()) + "\" size=\"" + Long.toString(fo.getFileSize()) + "\" execute=\"" + Boolean.toString(fo.getExecute()) + "\"/>\n");
 } catch (UnsupportedEncodingException uee) {
   System.err.println("UTF-8 unsupported!");
}
  }

  public static void writeFileFooter(final FileWriter output) throws IOException {
    output.write("</fileList>\n");
  }

  public static void writeFileList(final String filename, final ChannelList fileObjects) throws IOException {
    FileWriter output = new FileWriter(filename);
    // Write header
    writeFileHeader(output, fileObjects.getName());
    // Write files
    Iterator itr = fileObjects.getFiles().values().iterator();
    while (itr.hasNext()) {
      FileObject fo = (FileObject)itr.next();
      writeFileObject(output, fo);
    }
    // Write footer
    writeFileFooter(output);
    output.close();
  }
}
