// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 - 2006 Simon Goodall

package WFUT;

import java.io.*;
import javax.xml.parsers.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import java.net.*;

public class FileParser {
  public static class ProtoHandler extends DefaultHandler {

    private static final String ELEM_filelist = "fileList";
    private static final String ATTR_dir = "dir";
    private static final String ELEM_file = "file";
    private static final String ATTR_filename = "filename";
    private static final String ATTR_version = "version";
    private static final String ATTR_crc32 = "crc32";
    private static final String ATTR_size = "size";
    private static final String ATTR_execute = "execute";

    public void startElement(final String nameSpaceURI, final String localName,
        final String rawName, final Attributes atts) throws SAXException {

      if (rawName.equals(ELEM_filelist)) {
        for (int i = 0; i < atts.getLength(); ++i) {
          final String name = atts.getQName(i);
          final String value = atts.getValue(i);

          if (name.equals(ATTR_dir)) {
            // Set channel name
            fileMap.setName(value);
          }
        }
      } else if (rawName.equals(ELEM_file)) {
        fo = new FileObject();
        for (int i = 0; i < atts.getLength(); ++i) {
          final String name = atts.getQName(i);
          final String value = atts.getValue(i);

          if (name.equals(ATTR_filename)) {
            try {
              fo.setFilename(URLDecoder.decode(value, "UTF-8"));
              fileMap.addFile(fo);
            } catch (UnsupportedEncodingException uee) {
              System.err.println("UTF-8 unsupported!");
            }
          } else if (name.equals(ATTR_version)) {
            try {
              fo.setVersion(Integer.parseInt(value));
            } catch (NumberFormatException e) {
              fo.setVersion(-1);
            }
          } else if (name.equals(ATTR_crc32)) {
            try {
              fo.setCRC32(Long.parseLong(value));
            } catch (NumberFormatException e) {
              fo.setCRC32(-1);
            }
          } else if (name.equals(ATTR_size)) {
            try {
              fo.setFileSize(Long.parseLong(value));
            } catch (NumberFormatException e) {
              fo.setFileSize(-1);
            }
          } else if (name.equals(ATTR_execute)) {
            fo.setExecute(value.equals("true"));
          }
        }
      }
    }

    private FileObject fo = null;

    public ChannelList getFileMap() {
      return fileMap;
    }

    private ChannelList fileMap = new ChannelList();
  }

  public FileParser() throws SAXException, ParserConfigurationException {
    SAXParserFactory factory = SAXParserFactory.newInstance();
    factory.setValidating(false);
    factory.setNamespaceAware(false);
    SAXParser parser = factory.newSAXParser();
    // parser.setFeature("http://xml.org/sax/features/validation");
    reader = parser.getXMLReader();
    reader.setFeature("http://xml.org/sax/features/validation", false);
    reader.setFeature("http://xml.org/sax/features/namespaces", false);
    // reader.setFeature("http://xml.org/sax/features/namespaces", false);
    proto = new ProtoHandler();
    reader.setContentHandler(proto);
  }

  public void parse(InputStream is) throws IOException {
    try {
      reader.parse(new InputSource(is));
    } catch (SAXException e) {
      System.out.println(e);
    }
  }

  public void parse(String xmlString) throws Exception {
    reader.parse(new InputSource(new StringReader(xmlString)));
  }

  public ChannelList getFileMap() {
    return proto.getFileMap();
  }

  private XMLReader reader = null;

  private ProtoHandler proto = null;
}
